/****************************************************************************
**
** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
**
** This file is part of the tools applications of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License versions 2.0 or 3.0 as published by the Free Software
** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file.  Alternatively you may (at
** your option) use any later version of the GNU General Public
** License if such license has been publicly approved by Trolltech ASA
** (or its successors, if any) and the KDE Free Qt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
** you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** In addition, as a special exception, Trolltech, as the sole
** copyright holder for Qt Designer, grants users of the Qt/Eclipse
** Integration plug-in the right for the Qt/Eclipse Integration to
** link to functionality provided by Qt Designer and its related
** libraries.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
** granted herein.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

/*
  archiveextractor.cpp
*/

#include "archiveextractor.h"

QT_BEGIN_NAMESPACE

QList<ArchiveExtractor *> ArchiveExtractor::extractors;

/*!
  \class ArchiveExtractor
  
  \brief The ArchiveExtractor class is a base class for classes that
  know how to unpack a certain kind of archive file.

  The archive extractor contains a list of the filename extensions
  of the files that the archive extractor knows how to unpack.

  It maintains a static list of all the instances of ArchiveExtractor
  that have been created. It also has a static function for searching
  that list to find the archive extracter for a file with a certain
  extension.
 */

/*!
  The constructor takes a list of filename extensions, which it
  copies and saves internally. This archive extractor is prepended
  to the static list.
 */
ArchiveExtractor::ArchiveExtractor( const QStringList& extensions )
    : fileExts( extensions )
{
    extractors.prepend( this );
}

/*!
  The destructor deletes all the filename extensions.
 */
ArchiveExtractor::~ArchiveExtractor()
{
    extractors.removeAll( this );
}

/*!
  This function searches the static list of archive extractors
  to find the first one that can handle \a fileName. If it finds
  an acceptable extractor, it returns a pointer to it. Otherwise
  it returns null.
 */
ArchiveExtractor*
ArchiveExtractor::extractorForFileName( const QString& fileName )
{
    int dot = -1;
    while ( (dot = fileName.indexOf(QLatin1Char('.'), dot + 1)) != -1 ) {
        QString ext = fileName.mid( dot + 1 );
        QList<ArchiveExtractor *>::ConstIterator e = extractors.begin();
        while ( e != extractors.end() ) {
            if ( (*e)->fileExtensions().contains(ext) )
                return *e;
            ++e;
        }
    }
    return 0;
}

QT_END_NAMESPACE
